@@ -107,9 +107,17 @@ OC.L10N.register(
     "Select the board to link to a project" : "Izberite zbirko za povezavo s projektom",
     "Search by board title" : "Išči po imenu zbirke",
     "Select board" : "Izbor zbirke",
+    "Create a new card" : "Ustvari novo nalogo",
     "Select a board" : "Izbor zbirke",
-    "Select a card" : "Izbor naloge",
+    "Select a list" : "Izbor seznama",
+    "Card title" : "Naslov naloge",
     "Cancel" : "Prekliči",
+    "Creating the new card…" : "Poteka ustvarjanje nove naloge ...",
+    "\"{card}\" was added to \"{board}\"" : "Naloga »{card}« je dodana v zbirko »{board}«.",
+    "Open card" : "Odpri nalogo",
+    "Close" : "Zapri",
+    "Create card" : "Ustvari nalogo",
+    "Select a card" : "Izbor naloge",
     "Select the card to link to a project" : "Izbor naloge za povezavo do projekta",
     "Link to card" : "Poveži nalogo",
     "File already exists" : "Datoteka s tem imenom že obstaja.",
@@ -221,7 +229,6 @@ OC.L10N.register(
     "Archive card" : "Arhiviraj nalogo",
     "Delete card" : "Izbriši nalogo",
     "Move card to another board" : "Premakni nalogo v drugo zbirko",
-    "Select a list" : "Izbor seznama",
     "Card deleted" : "Naloga je izbrisana",
     "seconds ago" : "pred nekaj sekundami",
     "All boards" : "Vse zbirke",
@@ -258,6 +265,7 @@ OC.L10N.register(
     "upcoming cards" : "prihajajoče naloge",
     "Link to a board" : "Povezava do zbirke",
     "Link to a card" : "Povezava do naloge",
+    "Create a card" : "Ustvari nalogo",
     "Something went wrong" : "Prišlo je do napake ...",
     "Failed to upload {name}" : "Pošiljanje {name} je spodletelo",
     "Maximum file size of {size} exceeded" : "Omejitev velikosti datoteke {size} je prekoračena.",
